var bricks = window.bricks || {}

bricks.VideoBox = class extends bricks.JsWidget {
	create(){
		this.dom_element = this._create('video');
		this.set_attribute('autoplay', true);
		this.set_attribute('muted', true);
	}
	get_stream(){
		return this.stream;
	}
	set_stream(stream){
		this.stream = stream
		this.dom_element.srcObject = this.stream;
	}
}

bricks.RTCClient = class extends bricks.VBox {
	/*
	{
		"signaling_url":
		"ice_url":
		"info":{
			"id":
			"name":
		}
	}
	*/
	constructor(opts){
		opts.width = '100%';
		super(opts);
		this.build_head();
		this.videobox = new bricks.ResponsableBox({css:'filler'});
		this.localVideo = new bricks.VideoBox({css:'filler'});
		this.remoteVideo = new bricks.VideoBox({css:'filler'});
		this.videobox.add_widget(this.localVideo);
		this.videobox.add_widget(this.remoteVideo);
		this.add_widget(this.videobox);
		this.pc;
		this.onlineList= [];
		this.socket = new WebSocket(this.signaling_url);
		this.socket.onmessage = this.signaling_message_handle.bind(this);
		this.socket.onopen = this.signaling_login.bind(this);
		this.socket.onclose = null;
		this.socket.onerror = null;
	}
	build_head(){
		var box = new bricks.HBox({height:'50px'});
		this.add_widget(box);
		this.build_phone(box);
		this.peer_w = new bricks.Text({text:''});
		box.add_widget(this.peer_w);
	}
	build_phone(box){
		var opts = {
			states:[
				{
					state:'free',
					url:bricks_resource('imgs/free_phone.png')
				},{
					state:'using',
					url:bricks_resource('imgs/using_phone.png')
				}
			],
			state:'free',
			rate:2
		}
		this.phone = new bricks.StatedIcon(opts);
		this.phone.bind('click', this.phone_action.bind(this));
		box.add_widget(this.phone);
	}
	phone_action(){
		if (this.phone.state == 'free'){
			this.choose_peer();
		} else {
			this.call_close();
			this.phone.set_state('free');
		}
	}
	choose_peer(){
		var items = [];
		if (this.onlineList.length < 1){
			var w = new bricks.Error({title:'Error', 
				timeout:4,
				message:'no peer logined'
			});
			w.open();
			return;
		}
		this.onlineList.forEach( p => {
			var m = {
				name:p.id,
				label:p.name
			}
			items.push(m);
		});
		var menu = new bricks.Menu({
			width:'300px',
			height:'400px',
			items:items
		});
		menu.set_css('popup');
		bricks.Body.add_widget(menu);
		menu.bind('command', this.call_peer.bind(this));
		this.menu = menu;
	}
	async call_peer(event){
		console.log('event params=', event.params);
		bricks.Body.remove_widget(this.menu);
		var peer_info = {
			id:event.params.name,
			name:event.params.label
		}
		this.call_request(peer_info);
	}
	signaling_login(){
		var d = {
			type:'login',
			info:this.info
		}
		this.socket.send(JSON.stringify(d));
	}
	signaling_logout(){
		var d= {
			type:'logout',
			info:this.info
		}
		this.socket.send(JSON.stringify(d));
	}
	// 获取本地媒体流
	async getLocalStream() {
		try {
			const stream = await navigator.mediaDevices.getUserMedia({ video: true, audio: true });
			this.localVideo.set_stream(stream);
			return stream;
		} catch (error) {
			console.error('获取本地媒体流失败：', error);
		}
	}
	async call_request(peer_info){
		var d = {
			type:'callRequest',
			to:peer_info
		}		
		this.peer_info = peer_info;
		this.role = 'requester';
		this.socket.send(JSON.stringify(d));
	}
	async send_offer(){
		var offer = await this.pc.createOffer();
		await this.pc.setLocalDescription(offer);
		console.log('offer =', offer, this.pc.localDescription);
		this.socket.send(JSON.stringify({
			type:'offer',
			offer:this.pc.localDescription,
			to:this.peer_info
		}));
	}
	send_candidate(event){
		console.log('send_candidate(), event=', event);
		if (event.candidate) {
			this.socket.send(JSON.stringify({ 
		  		type: 'iceCandidate', 
				to:this.peer_info,
				candidate: event.candidate 
			}));
		}
	}
	async need_conform_call(peer_info){
		var w = new bricks.Conform({title:'请确认',
			message:'来自' + peer_info.name + '的连接请求'});
		w.bind('conformed', this.call_accepted.bind(this, peer_info));
		w.bind('cancelled', this.call_rejected.bind(this.peer_info));
		w.open();
	}
	async call_accepted(peer_info){
		if (this.peer_info){
			this.call_close();
		}
		this.peer_info = peer_info
		this.role = 'responser'
		await this.createPeerConnection(this.peer_info);
		this.socket.send(JSON.stringify({ 
			type: 'callAccepted', 
			to: peer_info 
		}));
	}
	async call_close(active){
		this.pc.onicecandidate = null;
		this.pc.ontrack = null;
		this.pc.close();
		if (! active){
			this.socket.send(JSON.stringify({
				type:'disconnect',
				to:this.peer_info
			}));
		}
		this.peer_info = null;
		this.localVideo.get_stream().getTracks().forEach(track => track.stop());
	}
	call_rejected(peer_info){
		this.socket.send(JSON.stringify({ 
			type: 'callRejected', 
			to: peer_info 
		}));
	}
	async signaling_message_handle(event){
		var datapkg = JSON.parse(event.data);
		var d = datapkg.data;
		console.log('data received from server', 'd=', d, d.type);
		switch (d.type){
			case 'onlineList':
				this.onlineList = d.onlineList;
				console.log('onlineList branch exe');
				break;
			case 'callRequest':
				this.need_conform_call(d.from);
				console.log('callRequest branch exe');
				break;
			case 'offer':
				var offer = new RTCSessionDescription(d.offer);
				await this.pc.setRemoteDescription(offer);
				var answer = await this.pc.createAnswer();
				await this.pc.setLocalDescription(answer);
				console.log('answer=', answer, 'desc=', this.pc.localDescription);
				this.socket.send(JSON.stringify({
					type:'answer',
					answer:this.pc.localDescription,
					to:this.peer_info
				}));
				console.log('offer branch exe');
				break;
			case 'answer':
				console.log('answer: d.from=', d.from, 'peer_info=', this.peer_info);
				if (d.from.id == this.peer_info.id){
					var desc = new RTCSessionDescription(d.answer);
					await this.pc.setRemoteDescription(desc);
					console.log('answer branch exe');
				}
				break;
			case 'iceCandidate':
				console.log('iceCandidate: d.from=', d.from, 'peer_info=', this.peer_info);
				if (d.from.id == this.peer_info.id){
					var candidate = new RTCIceCandidate(d.candidate);
					await this.pc.addIceCandidate(candidate);
					console.log('iceCandidate branch exe');
				}
				break;
			case 'callAccepted':
				await this.createPeerConnection(this.peer_info);
				console.log('callAccepted branch exe');
				break;
			case 'callRejected':
				this.peer_info = null;
				console.log('callRejected branch exe');
				break;
			case 'disconnect':
				this.call_close(true);
				console.log('disconnect branch exe');
				break;
			default:
				console.log(d.type, 'no branch defined');
				break;
		}
	}

	// 创建 PeerConnection
	async createPeerConnection(to_info) {
		this.phone.set_state('using');
		this.peer_w.set_text(to_info.name);
		if (! this.localVideo.get_stream()){
			await this.getLocalStream();
		}
		const configuration = {
		iceServers: [
			{ urls: this.ice_url },
			{ urls: 'stun:stunserver2024.stunprotocol.org:3478' }
		]
		};
		this.pc = new RTCPeerConnection(configuration);
		// 处理本地流添加到 PeerConnection
		this.localVideo
		.get_stream()
		.getTracks()
		.forEach(track => {
			this.pc.addTrack(track, this.localVideo.stream);
		});
		if (! this.pc){
			console.log('pc is null');
			return;
		}
		if (this.role == 'requester'){
			await this.send_offer();
		}

		// 处理 ICE 候选
		this.pc.onicecandidate = this.send_candidate.bind(this, to_info);

		// 处理远程流添加
		this.pc.ontrack = event => {
			this.remoteVideo.set_stream(event.streams[0]);
		};
		console.log('createPeerConnection() finished');
	}
}

bricks.Factory.register('RTCClient', bricks.RTCClient);
